##### CALCULATE Summed Positional Bigrams/Unigrams/Orthographic Legality #######
#Usage:
# python coltheartN.py <wordCorpus> <targetList> <cast lowercase: 0/1> <outfile>
#
# Output:
# <outfile> will contain the following information for each word in targetList,
# with each entry placed on a separate line:
#
# <word> <sumPosBi> <legalBi 1/0> <sumUni> <legalUni 1/0>'
#
# This program calculates token-based summed positional bigram frequencies and
# unigram (letter) frequencies, as well as reports whether a given string
# only contains unique unigrams and bigrams per those that are represented
# in <wordCorpus>.  Note that if using a list of unique words, e.g., such as
# those from a word frequency corpus, then this program will effectively
# calculate type as opposed to token statistics.  This program supports the
# use of non-English characters such as French accents, in which case these
# additional characters are treated in the same was a different letters.
#
# REQUIREMENTS: Python 2.7.x
#
# Author: Blair Armstrong, May, 2013

import sys

class bigramObj:
    def __init__(self, bi, nlet, pos):
        self.bi = bi
        self.nlet = nlet
        self.freq = 1
        self.pos = pos

    def __str__(self):
        return "nlet: %d %s: %d" % (self.nlet,self.bi,self.freq)

    def __repr__(self):
        return str(self)

    def __cmp__(self,other):
        if self.bi == other.bi and self.nlet == other.nlet \
                        and self.pos == other.pos:
            return 0
        else:
            return -1


class unigramObj:
    def __init__(self, uni, nlet, pos):
        self.uni = uni
        self.nlet = nlet
        self.freq = 1
        self.pos = pos

    def __str__(self):
        return "nlet: %d %s: %d" % (self.nlet,self.uni,self.freq)

    def __repr__(self):
        return str(self)

    def __cmp__(self,other):
        if self.uni == other.uni and self.nlet == other.nlet \
                        and self.pos == other.pos:
            return 0
        else:
            return -1


#count bigrams in given word in overall data
def countBigrams(word, biFreqs):

    #get each bigram in the word
    nlet = len(word)
    for i in range(0,len(word)-1):
        curBi = word[i:i+2]
        pos = i

        #see if it already exists; if it does, incr. freq, if not add bigram
        try:
            index = biFreqs.index(bigramObj(curBi,nlet,pos))
            biFreqs[index].freq = biFreqs[index].freq + 1
        except ValueError:
            biFreqs.append(bigramObj(curBi, nlet,pos))


def countUnigrams(word, uniFreqs):

    #get each bigram in the word
    nlet = len(word)
    for i in range(0,len(word)):
        curUni = word[i]
        pos = i

        #see if it already exists; if it does, incr. freq, if not add bigram
        try:
            index = uniFreqs.index(unigramObj(curUni,nlet,pos))
            uniFreqs[index].freq = uniFreqs[index].freq + 1
        except ValueError:
            uniFreqs.append(unigramObj(curUni, nlet,pos))



#calculate the constrained positional token bigram frequency for a word based
#on the <wordCorpus>
def sumBigrams(word,biFreqs):
    legal = 1;
    sum = 0
    nlet = len(word)
    for i in range(0,len(word)-1):
        curBi = word[i:i+2]
        pos = i

        #retrieve the count for the bigram
        try:
            index = biFreqs.index(bigramObj(curBi,nlet, pos))
            sum = sum + biFreqs[index].freq
        except ValueError:
            legal = 0;
            pass
            #no match; could include detection of illegal bigrams here

    return (sum,legal);


def sumUnigrams(word,uniFreqs):
    legal = 1;
    sum = 0
    nlet = len(word)
    for i in range(0,len(word)):
        curUni = word[i]
        pos = i

        #retrieve the count for the bigram
        try:
            index = uniFreqs.index(unigramObj(curUni,nlet, pos))
            sum = sum + uniFreqs[index].freq
        except ValueError:
            legal = 0;
            pass
            #no match; could include detection of illegal bigrams here

    return (sum,legal);


def main():

    #Processing Command Line Arguments
    if len(sys.argv) <= 4:
        print 'Usage: python posbigram.py <wordCorpus> <targetList> \
<cast lowercase: 0/1> <outFile>'
        sys.exit()
    else:
        for arg in sys.argv:
            print arg,
        print ''

    print 'Output format: <word> <sumPosBi> <legalBi 1/0> <sumUni> <legalUni 1/0>'

    cFile = open(sys.argv[1])
    tFile = open(sys.argv[2])
    toLower = int(sys.argv[3])
    outFile = open(sys.argv[4], "w")

    if toLower != 1 and toLower !=0:
        print 'Error: lowercase cast must be 0/1'
        sys.exit()

    #tabulate all of the bigrams in the corpus
    biFreqs = []
    uniFreqs = []

    word = cFile.readline()
    while word:
        word = word.strip()

        if toLower == 1:
            word = word.lower()

        #calculate bigrams for the word
        countBigrams(word,biFreqs)
        countUnigrams(word,uniFreqs)

        word = cFile.readline()
    cFile.close()


    #calculate posbigram frequencies for the target words
    word = tFile.readline()
    while word:
        word = word.strip()

        if toLower == 1:
            word = word.lower()

        summedBi,legalBi = sumBigrams(word, biFreqs)
        summedUni,legalUni = sumUnigrams(word,uniFreqs)
        outFile.write(word + "\t" + str(summedBi) + "\t" + str(legalBi) + "\t"
                        + str(summedUni) + "\t" + str(legalUni) + "\n");

        word =tFile.readline()
    tFile.close()


main()
